@@ -6,10 +6,11 @@ module Math.NumberTheory.ZetaBench
 
 import Test.Tasty.Bench
 
+import qualified Data.List.Infinite as Inf
 import Math.NumberTheory.Zeta
 
 benchSuite :: Benchmark
 benchSuite = bgroup "Zeta"
-  [ bench "riemann zeta"   $ nf (sum . take 20 . zetas) (1e-15 :: Double)
-  , bench "dirichlet beta" $ nf (sum . take 20 . betas) (1e-15 :: Double)
+  [ bench "riemann zeta"   $ nf (sum . Inf.take 20 . zetas) (1e-15 :: Double)
+  , bench "dirichlet beta" $ nf (sum . Inf.take 20 . betas) (1e-15 :: Double)
   ]
